'use strict';

var _config = require('./config');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Plugins should defined a config method that takes a joi object. By default
 * it should return a way to disallow config
 *
 * Config should be newed up with a joi schema (containing defaults via joi)
 *
 * let schema = { ... }
 * new Config(schema);
 *
 */

const data = {
  test: {
    hosts: ['host-01', 'host-02'],
    client: {
      type: 'datastore',
      host: 'store-01',
      port: 5050
    }
  }
};

const schema = _joi2.default.object({
  test: _joi2.default.object({
    enable: _joi2.default.boolean().default(true),
    hosts: _joi2.default.array().items(_joi2.default.string()),
    client: _joi2.default.object({
      type: _joi2.default.string().default('datastore'),
      host: _joi2.default.string(),
      port: _joi2.default.number()
    }).default(),
    undefValue: _joi2.default.string()
  }).default()
}).default();

describe('lib/config/config', function () {
  describe('class Config()', function () {

    describe('constructor', function () {

      it('should not allow any config if the schema is not passed', function () {
        const config = new _config.Config();
        const run = function () {
          config.set('something.enable', true);
        };
        expect(run).toThrow();
      });

      it('should allow keys in the schema', function () {
        const config = new _config.Config(schema);
        const run = function () {
          config.set('test.client.host', 'http://localhost');
        };
        expect(run).not.toThrow();
      });

      it('should not allow keys not in the schema', function () {
        const config = new _config.Config(schema);
        const run = function () {
          config.set('paramNotDefinedInTheSchema', true);
        };
        expect(run).toThrow();
      });

      it('should not allow child keys not in the schema', function () {
        const config = new _config.Config(schema);
        const run = function () {
          config.set('test.client.paramNotDefinedInTheSchema', true);
        };
        expect(run).toThrow();
      });

      it('should set defaults', function () {
        const config = new _config.Config(schema);
        expect(config.get('test.enable')).toBe(true);
        expect(config.get('test.client.type')).toBe('datastore');
      });
    });

    describe('#resetTo(object)', function () {

      let config;
      beforeEach(function () {
        config = new _config.Config(schema);
      });

      it('should reset the config object with new values', function () {
        config.set(data);
        const newData = config.get();
        newData.test.enable = false;
        config.resetTo(newData);
        expect(config.get()).toEqual(newData);
      });
    });

    describe('#has(key)', function () {

      let config;
      beforeEach(function () {
        config = new _config.Config(schema);
      });

      it('should return true for fields that exist in the schema', function () {
        expect(config.has('test.undefValue')).toBe(true);
      });

      it('should return true for partial objects that exist in the schema', function () {
        expect(config.has('test.client')).toBe(true);
      });

      it('should return false for fields that do not exist in the schema', function () {
        expect(config.has('test.client.pool')).toBe(false);
      });
    });

    describe('#set(key, value)', function () {
      let config;

      beforeEach(function () {
        config = new _config.Config(schema);
      });

      it('should use a key and value to set a config value', function () {
        config.set('test.enable', false);
        expect(config.get('test.enable')).toBe(false);
      });

      it('should use an object to set config values', function () {
        const hosts = ['host-01', 'host-02'];
        config.set({ test: { enable: false, hosts: hosts } });
        expect(config.get('test.enable')).toBe(false);
        expect(config.get('test.hosts')).toEqual(hosts);
      });

      it('should use a flatten object to set config values', function () {
        const hosts = ['host-01', 'host-02'];
        config.set({ 'test.enable': false, 'test.hosts': hosts });
        expect(config.get('test.enable')).toBe(false);
        expect(config.get('test.hosts')).toEqual(hosts);
      });

      it('should override values with just the values present', function () {
        const newData = _lodash2.default.cloneDeep(data);
        config.set(data);
        newData.test.enable = false;
        config.set({ test: { enable: false } });
        expect(config.get()).toEqual(newData);
      });

      it('should thow an exception when setting a value with the wrong type', function (done) {
        expect.assertions(2);

        const run = function () {
          config.set('test.enable', 'something');
        };

        try {
          run();
        } catch (err) {
          expect(err).toHaveProperty('name', 'ValidationError');
          expect(err.details[0].message).toBe('"enable" must be a boolean');
        }

        done();
      });
    });

    describe('#get(key)', function () {

      let config;

      beforeEach(function () {
        config = new _config.Config(schema);
        config.set(data);
      });

      it('should return the whole config object when called without a key', function () {
        const newData = _lodash2.default.cloneDeep(data);
        newData.test.enable = true;
        expect(config.get()).toEqual(newData);
      });

      it('should return the value using dot notation', function () {
        expect(config.get('test.enable')).toBe(true);
      });

      it('should return the clone of partial object using dot notation', function () {
        expect(config.get('test.client')).not.toBe(data.test.client);
        expect(config.get('test.client')).toEqual(data.test.client);
      });

      it('should throw exception for unknown config values', function () {
        const run = function () {
          config.get('test.does.not.exist');
        };
        expect(run).toThrowError(/Unknown config key: test.does.not.exist/);
      });

      it('should not throw exception for undefined known config values', function () {
        const run = function getUndefValue() {
          config.get('test.undefValue');
        };
        expect(run).not.toThrow();
      });
    });

    describe('#extendSchema(key, schema)', function () {
      let config;
      beforeEach(function () {
        config = new _config.Config(schema);
      });

      it('should allow you to extend the schema at the top level', function () {
        const newSchema = _joi2.default.object({ test: _joi2.default.boolean().default(true) }).default();
        config.extendSchema(newSchema, {}, 'myTest');
        expect(config.get('myTest.test')).toBe(true);
      });

      it('should allow you to extend the schema with a prefix', function () {
        const newSchema = _joi2.default.object({ test: _joi2.default.boolean().default(true) }).default();
        config.extendSchema(newSchema, {}, 'prefix.myTest');
        expect(config.get('prefix')).toEqual({ myTest: { test: true } });
        expect(config.get('prefix.myTest')).toEqual({ test: true });
        expect(config.get('prefix.myTest.test')).toBe(true);
      });

      it('should NOT allow you to extend the schema if somethign else is there', function () {
        const newSchema = _joi2.default.object({ test: _joi2.default.boolean().default(true) }).default();
        const run = function () {
          config.extendSchema('test', newSchema);
        };
        expect(run).toThrow();
      });
    });

    describe('#removeSchema(key)', function () {
      it('should completely remove the key', function () {
        const config = new _config.Config(_joi2.default.object().keys({
          a: _joi2.default.number().default(1)
        }));

        expect(config.get('a')).toBe(1);
        config.removeSchema('a');
        expect(() => config.get('a')).toThrowError('Unknown config key');
      });

      it('only removes existing keys', function () {
        const config = new _config.Config(_joi2.default.object());

        expect(() => config.removeSchema('b')).toThrowError('Unknown schema');
      });
    });
  });
});